package pwc.taxtech.atms.common;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Iterator;
import java.util.List;
import java.util.Optional;

public class POIUtil {
    private static final Logger logger = LoggerFactory.getLogger(POIUtil.class);
    public static final String XLSX = ".xlsx";
    public static final String XLS = ".xls";

    public static Optional<Workbook> cloneNewSheet(Sheet sheet, String fileName) {
        Workbook workbook;
        try {
            if (StringUtils.endsWith(fileName, XLSX)) {
                workbook = new XSSFWorkbook();
            } else if (StringUtils.endsWith(fileName, XLS)) {
                workbook = new HSSFWorkbook();
            } else {
                return Optional.empty();
            }
            Sheet tmpSheet = workbook.createSheet(sheet.getSheetName());
            cloneSheet(sheet, tmpSheet);
            return Optional.of(workbook);
        } catch (Exception e) {
            logger.error("cloneNewSheet error.", e);
        }
        return Optional.empty();
    }

    public static void cloneSheet(Sheet sheet, Sheet targetSheet) {
        for (int r = sheet.getFirstRowNum(); r <= sheet.getLastRowNum(); r++) {
            Row row = sheet.getRow(r);
            if (null == row) {
                continue;
            }
            Row targetRow = targetSheet.createRow(r);
            for (int c = row.getFirstCellNum(); c <= row.getLastCellNum(); c++) {
                if(c < 0 ){
                    continue;
                }
                Cell cell = row.getCell(c);
                if (null == cell) {
                    continue;
                }
                Cell targetCell = targetRow.createCell(c);
                targetCell.setCellType(cell.getCellTypeEnum());
                switch (cell.getCellTypeEnum()) {
                    case STRING:
                        targetCell.setCellValue(cell.getStringCellValue());
                        break;
                    case NUMERIC:
                        targetCell.setCellValue(cell.getNumericCellValue());
                        break;
                    case BOOLEAN:
                        targetCell.setCellValue(cell.getBooleanCellValue());
                        break;
                    case FORMULA:
                        targetCell.setCellFormula(cell.getCellFormula());
                        break;
//                    case ERROR:
////                    case BLANK:
////                    case _NONE:
                    default:
                        break;
                }
                if (null != cell.getCellComment()) {
                    targetCell.setCellComment(cell.getCellComment());
                }
                if (null != cell.getCellStyle() && targetCell.getCellStyle() != null ) {
                    try{
                        targetCell.getCellStyle().cloneStyleFrom(cell.getCellStyle());
                    }catch (Exception e){
                        //e.printStackTrace();
                    }
                }
                if (null != cell.getHyperlink()) {
                    targetCell.setHyperlink(cell.getHyperlink());
                }
            }
        }
    }

    public static void cloneSheetAndStyle(Sheet sheet, Sheet targetSheet,Workbook tWorkbook) {
        //设置合并单元格
        List<CellRangeAddress> merges =  sheet.getMergedRegions();
        if(CollectionUtils.isNotEmpty(merges)){
            for(CellRangeAddress merge : merges){
                targetSheet.addMergedRegion(merge);
            }
        }
        for (int r = sheet.getFirstRowNum(); r <= sheet.getLastRowNum(); r++) {
            Row row = sheet.getRow(r);
            if (null == row) {
                continue;
            }
            Row targetRow = targetSheet.createRow(r);
            for (int c = row.getFirstCellNum(); c <= row.getLastCellNum(); c++) {
                if(c < 0 ){
                    continue;
                }
                Cell cell = row.getCell(c);
                if (null == cell) {
                    continue;
                }
                Cell targetCell = targetRow.createCell(c);
                targetCell.setCellType(cell.getCellTypeEnum());
                switch (cell.getCellTypeEnum()) {
                    case STRING:
                        if (cell.getStringCellValue().indexOf("KeyIn")>0){
                            targetCell.setCellValue("");
                        }else{
                            targetCell.setCellValue(cell.getStringCellValue());
                        }
                        break;
                    case NUMERIC:
                        targetCell.setCellValue(cell.getNumericCellValue());
                        break;
                    case BOOLEAN:
                        targetCell.setCellValue(cell.getBooleanCellValue());
                        break;
                    case FORMULA:
                        targetCell.setCellFormula(cell.getCellFormula());
                        break;
//                    case ERROR:
////                    case BLANK:
////                    case _NONE:
                    default:
                        break;
                }
                if (null != cell.getCellComment()) {
                    targetCell.setCellComment(cell.getCellComment());
                }
                if (null != cell.getCellStyle()) {
                    CellStyle newstyle=tWorkbook.createCellStyle();
                    targetCell.setCellStyle(newstyle);
                    try{
                        targetCell.getCellStyle().cloneStyleFrom(cell.getCellStyle());
                    }catch (Exception e){
                        //e.printStackTrace();
                    }
                }
                if (null != cell.getHyperlink()) {
                    targetCell.setHyperlink(cell.getHyperlink());
                }
            }
        }
    }
    public static String getCellFormulaString(Cell cell) {
        switch (cell.getCellTypeEnum()) {
            case STRING:
                return cell.getStringCellValue();
            case FORMULA:
                return cell.getCellFormula();
            case BLANK:
                return cell.getStringCellValue();
            case NUMERIC:
                return String.valueOf(cell.getNumericCellValue());
            case BOOLEAN:
                return String.valueOf(cell.getBooleanCellValue());
            default:
                return StringUtils.EMPTY;
        }
    }

    public static Optional<String> getFileSuffix(String fileName) {
        if (StringUtils.endsWith(fileName, XLSX)) {
            return Optional.of(XLSX);
        } else if (StringUtils.endsWith(fileName, XLS)) {
            return Optional.of(XLS);
        } else {
            return Optional.empty();
        }
    }

    public static Row createRow(Sheet sheet, Integer rowIndex) {
        Row row = null;
        if (sheet.getRow(rowIndex) != null) {
            int lastRowNo = sheet.getLastRowNum();
            sheet.shiftRows(rowIndex, lastRowNo, 1);
        }
        row = sheet.createRow(rowIndex);
        return row;
    }

    public static Row createAndCloneRow(Workbook wb , Sheet sheet, Integer destRowIndex, Row fromRow) {
        Row toRow = null;
        if (sheet.getRow(destRowIndex) != null) {
            int lastRowNo = sheet.getLastRowNum();
            sheet.shiftRows(destRowIndex, lastRowNo, 1);
        }
        toRow = sheet.createRow(destRowIndex);
        for (Iterator cellIt = fromRow.cellIterator(); cellIt.hasNext();) {
            Cell tmpCell = (Cell) cellIt.next();
            Cell newCell = toRow.createCell(tmpCell.getColumnIndex());
            copyCell(wb, tmpCell, newCell, false);
        }
        return toRow;
    }

    public static void copyCell(Workbook wb,Cell srcCell, Cell distCell,
                                boolean copyValueFlag) {
        CellStyle newstyle=wb.createCellStyle();
        newstyle.cloneStyleFrom(srcCell.getCellStyle());
//        copyCellStyle(wb,srcCell.getCellStyle(), newstyle);
//        distCell.setEncoding(srcCell.getEncoding());
        //样式
        distCell.setCellStyle(newstyle);
        //评论
        if (srcCell.getCellComment() != null) {
            distCell.setCellComment(srcCell.getCellComment());
        }
        // 不同数据类型处理
        int srcCellType = srcCell.getCellType();
        distCell.setCellType(srcCellType);
        if (copyValueFlag) {
            if (srcCellType == Cell.CELL_TYPE_NUMERIC) {
                if (DateUtil.isCellDateFormatted(srcCell)) {
                    distCell.setCellValue(srcCell.getDateCellValue());
                } else {
                    distCell.setCellValue(srcCell.getNumericCellValue());
                }
            } else if (srcCellType == Cell.CELL_TYPE_STRING) {
                distCell.setCellValue(srcCell.getRichStringCellValue());
            } else if (srcCellType == Cell.CELL_TYPE_BLANK) {
                // nothing21
            } else if (srcCellType == Cell.CELL_TYPE_BOOLEAN) {
                distCell.setCellValue(srcCell.getBooleanCellValue());
            } else if (srcCellType == Cell.CELL_TYPE_ERROR) {
                distCell.setCellErrorValue(srcCell.getErrorCellValue());
            } else if (srcCellType == Cell.CELL_TYPE_FORMULA) {
                distCell.setCellFormula(srcCell.getCellFormula());
            } else { // nothing29
            }
        }
    }

    public static void copyCellStyle(Workbook wb,CellStyle fromStyle,
                                     CellStyle toStyle) {
        toStyle.setAlignment(HorizontalAlignment.forInt(fromStyle.getAlignment()));
        //边框和边框颜色
        toStyle.setBorderBottom(BorderStyle.valueOf(fromStyle.getBorderBottom()));
        toStyle.setBorderLeft(BorderStyle.valueOf(fromStyle.getBorderLeft()));
        toStyle.setBorderRight(BorderStyle.valueOf(fromStyle.getBorderRight()));
        toStyle.setBorderTop(BorderStyle.valueOf(fromStyle.getBorderTop()));
        toStyle.setTopBorderColor(fromStyle.getTopBorderColor());
        toStyle.setBottomBorderColor(fromStyle.getBottomBorderColor());
        toStyle.setRightBorderColor(fromStyle.getRightBorderColor());
        toStyle.setLeftBorderColor(fromStyle.getLeftBorderColor());

        //背景和前景
        toStyle.setFillBackgroundColor(fromStyle.getFillBackgroundColor());
        toStyle.setFillForegroundColor(fromStyle.getFillForegroundColor());
//
        toStyle.setDataFormat(fromStyle.getDataFormat());
        toStyle.setFillPattern(FillPatternType.forInt(fromStyle.getFillPattern()));
		toStyle.setFont(wb.getFontAt(fromStyle.getFontIndex()));
        toStyle.setHidden(fromStyle.getHidden());
        toStyle.setIndention(fromStyle.getIndention());//首行缩进
        toStyle.setLocked(fromStyle.getLocked());
        toStyle.setRotation(fromStyle.getRotation());//旋转
        toStyle.setVerticalAlignment(VerticalAlignment.forInt(fromStyle.getVerticalAlignment()));
        toStyle.setWrapText(fromStyle.getWrapText());

    }


}
