package pwc.taxtech.atms.controller;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.dto.periodReport.PeriodReportDataParam;
import pwc.taxtech.atms.dto.vatdto.ManualDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.PeriodJobDto;
import pwc.taxtech.atms.dto.vatdto.ReportDataDto;
import pwc.taxtech.atms.service.impl.CitReportServiceImpl;
import pwc.taxtech.atms.service.impl.CitReportUploadService;
import pwc.taxtech.atms.service.impl.ReportUploadService;
import pwc.taxtech.atms.vat.entity.PeriodJob;
import pwc.taxtech.atms.vat.entity.ReportUpload;
import pwc.taxtech.atms.vat.service.impl.ReportServiceImpl;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.util.List;
import java.util.Optional;

import static javax.servlet.http.HttpServletResponse.SC_NO_CONTENT;
import static javax.servlet.http.HttpServletResponse.SC_OK;

/**
 * @author zhikai.z.wei
 * @description CIT报表生成（数据处理）
 */
@RestController
@RequestMapping(value = "/api/v1/citReport/")
public class CitReportController {
    private static Logger logger = LoggerFactory.getLogger(CitReportController.class);

    @Autowired
    CitReportServiceImpl reportService;
    @Autowired
    CitReportServiceImpl citReportService;
    @Autowired
    CitReportUploadService citReportUploadService;

    /**
     * 获取CIT所有要生成的报表模板相关信息，
     * 注意CIT不存在月份期间,默认为0
     * @param projectId
     * @param serviceType
     * @return
     */
    @RequestMapping(value = "citTemplate/{projectId}/{serviceType}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getCitTemplate(@PathVariable String projectId, @PathVariable int serviceType) {
        logger.info("获取CIT所有要生成的报表模板相关信息，参数");
        return citReportService.getReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType),0);
    }

    /**
     * 生成CIT所有的报表
     * @param projectId
     * @param generator
     * @param mergeManual
     * @return
     */
    @RequestMapping(value = "generateByTotal/{projectId}/{mergeManual}", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity generateAllData(@PathVariable String projectId,
                                          @RequestParam Optional<String> generator, @PathVariable Boolean mergeManual) {
        return ResponseEntity.ok(citReportService.generateCitData(projectId, EnumServiceType.CIT, mergeManual,0,null, generator));
    }

    /**
     * 生成CIT总分机构分配表
     * @return
     */
    @RequestMapping(value = "generateDistributionTable", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto generateDistributionTable(@RequestBody CitDistributionDto citDistributionDto) {
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("生成成功");
            apiResultDto.setData(citReportService.generateTotalBranchOrgDisTable(citDistributionDto.getProjectId()));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("生成失败");
            return apiResultDto;
        }
    }

    /**
     * 生成CIT固资损失计算
     * @return
     */
    @RequestMapping(value = "generateAssetEamMapping", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto generateAssetEamMapping(@RequestBody CitAssetsListDto citAssetsListDto) {
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("生成成功");
            apiResultDto.setData(citReportService.generateAssetEamMapping(citAssetsListDto));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("生成失败");
            return apiResultDto;
        }
    }

    /**
     * 获取当前卡片所拥有的模板
     * @param projectId
     * @param serviceType
     * @param period
     * @return
     */
    @RequestMapping(value = "filterTemplate/{projectId}/{serviceType}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getFilterTemplate(@PathVariable String projectId, @PathVariable int serviceType, @PathVariable Integer period) {
        return citReportService.getFilterReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType), period);
    }

    /**
     * 获取当前卡片的线程任务
     * @param projectId
     * @param period
     * @return
     */
    @RequestMapping(value = "getPeriodJob/{projectId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    public PeriodJobDto getPeriodJob(@PathVariable String projectId, @PathVariable Integer period) {
        PeriodJob job = citReportService.getPeriodJob(projectId, period);
        if (job != null) {
            return new PeriodJobDto().copyFromPeriodJob(job);
        } else {
            return null;
        }
    }

    /**
     * 获取固定资产及EAM Mapping的数据
     * @param citAssetsListDto
     * @return
     */
    @RequestMapping(value = "/getAssetEamMappingsPage", method = RequestMethod.POST)
    public @ResponseBody
    ApiResultDto getAssetEamMappingsPage(@RequestBody CitAssetsListDto citAssetsListDto){
        logger.info("获取固定资产及EAM Mapping的数据");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(citReportService.getAssetEamMappingPage(citAssetsListDto));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }


    /**
     * 固定资产及EAM Mapping导出
     * @param paras
     * @param response
     */
    @RequestMapping(value = "exportAEMData", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void exportAEMData(@RequestBody CitAssetsListDto paras, HttpServletResponse response) {
        int count = citReportService.exportAEMData2(paras, response);
        if (count == 0) {
            response.setStatus(SC_NO_CONTENT);
        } else {
            response.setStatus(SC_OK);
        }

    }

    /**
     * 获取  企业所得税汇总纳税分支机构所得税分配表（A202000）
     * @param citDistributionDto
     * @return
     */
    @RequestMapping(value = "/getDistributionTables", method = RequestMethod.POST)
    public @ResponseBody
    ApiResultDto getCitDistribution(@RequestBody CitDistributionDto citDistributionDto){
        logger.info("获取 企业所得税汇总纳税分支机构所得税分配表 的数据");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(citReportService.getCitDistribution(citDistributionDto));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }
    /**
     * 固定资产及EAM Mapping导出
     * @param paras
     * @param response
     */
    @RequestMapping(value = "exportDTData", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void exportDTData(@RequestBody CitDistributionDto paras, HttpServletResponse response) {
        int count = citReportService.exportDTData2(paras, response);
        if (count == 0) {
            response.setStatus(SC_NO_CONTENT);
        } else {
            response.setStatus(SC_OK);
        }

    }
    /**
     * 获取单元格相关数据
     * @param reportId
     * @param from
     * @return
     */
    @RequestMapping(value = "reportData/{reportId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<ReportDataDto> getReportData(@PathVariable Long reportId, @RequestHeader String from) {
        OperationResultDto resultDto = new OperationResultDto();
        if (reportId == null || reportId == 0L) {
            resultDto.setResult(false);
            return resultDto;
        }
        return reportService.getCellData(reportId, from);
    }

    /**
     * 添加手工数据源
     * @param data
     * @param from
     * @return
     */
    @RequestMapping(value = "addCellManualData", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity addCellManualDataSource(@RequestBody ManualDataSourceDto data, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return ResponseEntity.ok(reportService.addCellManualDataSource(data, from));
    }

    @RequestMapping(value = "uploadReportData/{projectId}/{period}", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    public OperationResultDto uploadReportData(@RequestBody PeriodReportDataParam param,
                                               @PathVariable(name = "period", required = false) Integer period,
                                               @PathVariable(name = "projectId", required = false) String projectId) {
        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        try {
            Workbook tWorkbook = citReportService.generateReportData(param.getReportIds(), projectId, period);
            tWorkbook.write(bout);
            ReportUpload data = new ReportUpload();
            data.setProjectId(projectId);
            data.setPeriod(period);
            citReportUploadService.saveData(bout.toByteArray(), data);
            return OperationResultDto.success();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (bout != null) {
                    bout.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
        return OperationResultDto.error(ErrorMessage.SystemError);
    }

}