package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumCitImportType;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dto.CitDataImportLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ServiceException;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class CitImportExcelServiceImpl extends BaseService {

    @Resource
    private OrganizationMapper organizationMapper;

    @Autowired
    private CitDataImportLogMapper citDataImportLogMapper;

    @Autowired
    private CitJournalEntryAdjustMapper citJournalEntryAdjustMapper;

    @Autowired
    private CitTrialBalanceMapper citTrialBalanceMapper;

    @Autowired
    private CitDraftAccountMappingMapper citDAMappingMapper;

    @Autowired
    private CitBalanceSheetPrcAdjustMapper citBSPrcAdjustMapper;

    @Autowired
    private CitProfitPrcAdjustMapper citProfitPrcAdjustMapper;

    @Autowired
    private CitEAMAssetsDisposalMapper citEAMAssetsMapper;

    @Autowired
    private CitSalaryAdvanceMapper citSalaryAdvanceMapper;

    @Autowired
    private FileService fileService;

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private CitTbamMapper citTbamMapper;

    /**
     * 根据导入的文件类型进行方法分发
     *
     * @param file
     * @param fileName
     * @param periodDate
     * @param importType
     * @param importFileType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto citImportExcel(MultipartFile file,List<String> orgList, String fileName, String periodDate, Integer importType, Integer importFileType, Integer month) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto operationResultDto = new OperationResultDto();
        InputStream input = file.getInputStream();

        //判断期间参数是否为空等
        if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
            throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
        }
        //转换期间为整型
        Integer period = DateUtils.strToPeriodY(periodDate);

        //获取上传文件类型的枚举值
        EnumCitImportType enumValue = EnumCitImportType.valueOf(importFileType);
        switch (enumValue) {
            case JournalAdjust:
                operationResultDto = importJournalAdjust(input, orgList, fileName, period, importType);
                break;
            case TrialBalance:
                operationResultDto = importTrialBalance(input, orgList, fileName, period, importType);
                break;
            case DraftAccountMapping:
                operationResultDto = importDraftAccountMapping(input, orgList, fileName, period, importType);
                break;
            case BalanceSheetPrcAdjust:
                operationResultDto = importBalanceSheetPrcAdjust(input, orgList, fileName, period, importType);
                break;
            case ProfitPrcAdjust:
                operationResultDto = importProfitPrcAdjust(input, orgList, fileName, period, importType);
                break;
            case EAMAssetsDisposal:
                operationResultDto = importEAMAssetsDisposal(input, orgList, fileName, period, importType);
                break;
            case SalaryAdvance:
                //目前只有预提费用需要月份
//                Integer month = DateUtils.strToPeriodM(periodDate);
                operationResultDto = importSalaryAdvance(input, orgList, fileName, period, month, importType);
                break;
        }
        return operationResultDto;
    }

    /**
     * 导入日记账调整版
     *
     * @param inputStream
     * @param fileName
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importJournalAdjust(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citJournalAdjust");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前日记账调整版导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitJournalEntryAdjust> journalAdjustList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
        String companyName = CitCommonUtil.getValue(sheet.getRow(1).getCell(19)).toString();
        String companyCode = CitCommonUtil.getValue(sheet.getRow(1).getCell(9)).toString();

        if("".equals(companyCode)){
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }

        companyCode = companyCode.contains(".")?companyCode.substring(0,companyCode.indexOf(".")):companyCode;

        //根据code（主体）获取机构相关信息
        List<Organization> organizations = getOrganizationByCode(companyCode);
        String orgId = "";
        Project project = null;
        String taxPayerId = "";
        if (organizations != null && organizations.size() > 0) {
            orgId = organizations.get(0).getId();
            project = getProject(orgId,period);
            taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                    organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();

            if(!orgList.contains(orgId)){
                saveResult.setResult(false);
                saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
                return saveResult;
            }
            if(project == null){
                saveResult.setResult(false);
                saveResult.setResultMsg(ErrorMessageCN.NoProjectError);
                return saveResult;
            }
        }else{
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }

        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 1; rowNum <= sheet.getLastRowNum(); rowNum++) {
            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.JournalAdjust.getCode(), period,
                    "", "日记账调整版", false);
            citDataImportLogError.setRecordSize(1);
            //获取该行数据
            Row rowData = sheet.getRow(rowNum);
            if(rowData == null){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"整行为空");
                citDataImportLogList.add(citDataImportLogError);
            }
            //拼接日记账调整版的实体
            CitJournalEntryAdjust citJournal = new CitJournalEntryAdjust();
            citJournal.setId(idService.nextId());
            citJournal.setProjectId(project.getId());
            citJournal.setOrganizationId(orgId);
            citJournal.setPeriod(period);
            citJournal.setSource("人工导入");
            citJournal.setCreatedBy(currentUserName);

            citJournal.setApprovalStatus(CitCommonUtil.getValue(rowData.getCell(0)).toString());
            citJournal.setPostedStatus(CitCommonUtil.getValue(rowData.getCell(1)).toString());
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Object cellValue = CitCommonUtil.getValue(rowData.getCell(2));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行会计期间为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }

            citJournal.setAccountPeriod(new Integer(cellValue.toString().replace("-", "")));
            citJournal.setAccountingDate(sdf.parse(CitCommonUtil.getValue(rowData.getCell(3)).toString()));
            citJournal.setJournalSource(CitCommonUtil.getValue(rowData.getCell(4)).toString());
            citJournal.setCategory(CitCommonUtil.getValue(rowData.getCell(5)).toString());
            citJournal.setName(CitCommonUtil.getValue(rowData.getCell(6)).toString());
            citJournal.setVoucherNum(CitCommonUtil.getValue(rowData.getCell(7)).toString());
            citJournal.setDescription(CitCommonUtil.getValue(rowData.getCell(8)).toString());
            cellValue = CitCommonUtil.getValue(rowData.getCell(9));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行主体为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citJournal.setOrgCode(cellValue.toString().replace(".0",""));
            citJournal.setSegment2(CitCommonUtil.getValue(rowData.getCell(10)).toString());
            cellValue = CitCommonUtil.getValue(rowData.getCell(11));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行科目代码为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citJournal.setSubjectCode(cellValue.toString());
            citJournal.setSegment4(CitCommonUtil.getValue(rowData.getCell(12)).toString());
            citJournal.setSegment5(CitCommonUtil.getValue(rowData.getCell(13)).toString().replace(".0",""));
            citJournal.setSegment6(CitCommonUtil.getValue(rowData.getCell(14)).toString().replace(".0",""));
            citJournal.setSegment7(CitCommonUtil.getValue(rowData.getCell(15)).toString().replace(".0",""));
            citJournal.setSegment8(CitCommonUtil.getValue(rowData.getCell(16)).toString());
            citJournal.setSegment9(CitCommonUtil.getValue(rowData.getCell(17)).toString());
            citJournal.setSegment10(CitCommonUtil.getValue(rowData.getCell(18)).toString());
            citJournal.setOrgName(CitCommonUtil.getValue(rowData.getCell(19)).toString());
            citJournal.setSegment2Name(CitCommonUtil.getValue(rowData.getCell(20)).toString());
            citJournal.setSubjectName(CitCommonUtil.getValue(rowData.getCell(21)).toString());
            citJournal.setSegment4Name(CitCommonUtil.getValue(rowData.getCell(22)).toString());
            citJournal.setSegment5Name(CitCommonUtil.getValue(rowData.getCell(23)).toString());
            citJournal.setSegment6Name(CitCommonUtil.getValue(rowData.getCell(24)).toString());
            citJournal.setSegment7Name(CitCommonUtil.getValue(rowData.getCell(25)).toString());
            citJournal.setSegment8Name(CitCommonUtil.getValue(rowData.getCell(26)).toString());
            citJournal.setSegment9Name(CitCommonUtil.getValue(rowData.getCell(27)).toString());
            citJournal.setSegment10Name(CitCommonUtil.getValue(rowData.getCell(28)).toString());
            citJournal.setJournalCurrencyCode(CitCommonUtil.getValue(rowData.getCell(29)).toString());
            citJournal.setSobCurrencyCode(CitCommonUtil.getValue(rowData.getCell(30)).toString());
            //借方金额
            cellValue = CitCommonUtil.getValue(rowData.getCell(31));
            if("".equals(cellValue)){
                cellValue = "0";
            }
            citJournal.setAccountedDr(new BigDecimal(cellValue.toString()));
            //贷方金额
            cellValue = CitCommonUtil.getValue(rowData.getCell(32));
            if("".equals(cellValue)){
                cellValue = "0";
            }
            citJournal.setAccountedCr(new BigDecimal(cellValue.toString()));
            cellValue = CitCommonUtil.getValue(rowData.getCell(33));
            citJournal.setEnteredDr(new BigDecimal("".equals(cellValue)?"0":cellValue.toString()));
            cellValue = CitCommonUtil.getValue(rowData.getCell(34));
            citJournal.setEnteredCr(new BigDecimal("".equals(cellValue)?"0":cellValue.toString()));
            citJournal.setCfItem(CitCommonUtil.getValue(rowData.getCell(35)).toString());
            citJournal.setAttribute1(CitCommonUtil.getValue(rowData.getCell(36)).toString());
            cellValue = CitCommonUtil.getValue(rowData.getCell(37));
            if(!"".equals(cellValue)){
                citJournal.setAttribute2(sdf.parse(cellValue.toString()));
            }
            citJournal.setAttribute3(CitCommonUtil.getValue(rowData.getCell(38)).toString());
            citJournal.setAttribute4(CitCommonUtil.getValue(rowData.getCell(39)).toString());
            citJournal.setAttribute5(CitCommonUtil.getValue(rowData.getCell(40)).toString().replace(".0",""));
            citJournal.setAttribute6(CitCommonUtil.getValue(rowData.getCell(41)).toString().replace(".0",""));
            citJournal.setAttribute7(CitCommonUtil.getValue(rowData.getCell(42)).toString().replace(".0",""));
            citJournal.setAttribute8(CitCommonUtil.getValue(rowData.getCell(43)).toString());
            citJournal.setAttribute9(CitCommonUtil.getValue(rowData.getCell(44)).toString());
            citJournal.setAttribute10(CitCommonUtil.getValue(rowData.getCell(45)).toString());
            citJournal.setAttribute11(CitCommonUtil.getValue(rowData.getCell(46)).toString());
            citJournal.setAttribute12(CitCommonUtil.getValue(rowData.getCell(47)).toString());
            citJournal.setAttribute13(CitCommonUtil.getValue(rowData.getCell(48)).toString());
            citJournal.setAttribute14(CitCommonUtil.getValue(rowData.getCell(49)).toString());
            citJournal.setAttribute15(CitCommonUtil.getValue(rowData.getCell(50)).toString());
            citJournal.setAttribute16(CitCommonUtil.getValue(rowData.getCell(51)).toString());
            journalAdjustList.add(citJournal);
        }

        //判断导入的方式
        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            //覆盖导入要删除原有的该操作人在当前期间导入的数据，且更改之前导入的日志记录状态
            msg = "覆盖导入成功";
            CitJournalEntryAdjustExample citJEAExample = new CitJournalEntryAdjustExample();
            citJEAExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(project.getId());
            logger.info("删除符合条件的日记账");
            citJournalEntryAdjustMapper.deleteByExample(citJEAExample);
            //删除自动生成的试算平衡表相关数据
            CitTrialBalanceExample citTbExample = new CitTrialBalanceExample();
            citTbExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(project.getId());
            int deleteByExample = citTrialBalanceMapper.deleteByExample(citTbExample);
            logger.info("删除符合条件的试算平衡表数据");
            CitTbamExample citTbamExample = new CitTbamExample();
            citTbamExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(project.getId());
            logger.info("删除符合条件的试算平衡表Mapping数据");
            citTbamMapper.deleteByExample(citTbamExample);
            updateImportLog(EnumCitImportType.JournalAdjust.getCode());
        }
        int insertBatchNum = citJournalEntryAdjustMapper.insertBatch(journalAdjustList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                EnumCitImportType.JournalAdjust.getCode(), period,
                companyName, "日记账调整",true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);

        citDataImportLogList.add(citDataImportLog);
        //导入日志录入到数据库
        addDataImportLog(citDataImportLogList);
        //根据日记账自动生成试算平衡表，即试算平衡表生成版
        List<String> orgListTemp = getOrgList(project);
        autoGeneTB(project, orgListTemp,currentUserName);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入试算平衡表
     *
     * @param inputStream
     * @param fileName
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importTrialBalance(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitTrialBalance> trialBalanceList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
        //获取当前期间
//        Integer period = CitCommonUtil.getPeriod();

        Map<String, Integer> companySuccessMap = new HashMap();
        Map<String, Integer> companyFailMap = new HashMap();
        Map<String, Integer> noCompanyMap = new HashMap();
        Map<String,Integer> noProjectMap = new HashMap<>();
        List<String> projectIdList = new ArrayList<>();
        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 8; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitTrialBalance trialBalance = new CitTrialBalance();
            trialBalance.setId(idService.nextId());
            trialBalance.setPeriod(period);
            trialBalance.setCreateBy(currentUserName);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.TrialBalance.getCode(), period,
                    "", "试算平衡表", false);
            citDataImportLogError.setRecordSize(1);

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(5));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行主体为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }

            String companyCode = cellValue.toString().replace(".0","");
            //根据code（主体）获取机构相关信息
            List<Organization> organizations = getOrganizationByCode(companyCode);
            String companyName = "";
            String orgId = "";
            Project project = null;
            String taxPayerId = "";
            if (organizations != null && organizations.size() > 0) {
                orgId = organizations.get(0).getId();
                project = getProject(orgId, period);
                taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                        organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                companyName = organizations.get(0).getName();

                //拼接公司相关信息，作为键，其Excel出现的数量作为值
                StringBuilder sb = new StringBuilder(companyCode);
                sb.append(",");
                sb.append(orgId);
                sb.append(",");
                sb.append(taxPayerId);
                sb.append(",");
                sb.append(companyName);
                if(!orgList.contains(orgId)){
                    companyFailMap.put(sb.toString(),companyFailMap.containsKey(sb.toString())?companyFailMap.get(sb.toString())+1:1);
                    continue;
                }
                if(project == null){
                    noProjectMap.put(sb.toString(),noProjectMap.containsKey(sb.toString())?noProjectMap.get(sb.toString())+1:1);
                    continue;
                }
                projectIdList.add(project.getId());
                companySuccessMap.put(sb.toString(),companySuccessMap.containsKey(sb.toString())?companySuccessMap.get(sb.toString())+1:1);

            }else{
                noCompanyMap.put(companyCode,noCompanyMap.containsKey(companyCode)?noCompanyMap.get(companyCode)+1:1);
                continue;
            }

            trialBalance.setOrganizationId(orgId);
            trialBalance.setProjectId(project.getId());
            trialBalance.setAccountCategory(CitCommonUtil.getValue(rowData.getCell(0)).toString());
            trialBalance.setAcctCode1(CitCommonUtil.getValue(rowData.getCell(1)).toString());
            trialBalance.setAcctName1(CitCommonUtil.getValue(rowData.getCell(2)).toString());
            trialBalance.setAcctName2(CitCommonUtil.getValue(rowData.getCell(3)).toString());
            trialBalance.setAcctName3(CitCommonUtil.getValue(rowData.getCell(4)).toString());
            trialBalance.setSegment1(companyCode);
            trialBalance.setSegment2(CitCommonUtil.getValue(rowData.getCell(6)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(7));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行科目为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
//            trialBalance.setAccountCode(cellValue.toString().replace(".0",""));
            trialBalance.setAccountCode(new BigDecimal(cellValue.toString()).toPlainString());
            trialBalance.setSegment4(CitCommonUtil.getValue(rowData.getCell(8)).toString());
            trialBalance.setSegment5(CitCommonUtil.getValue(rowData.getCell(9)).toString());
            trialBalance.setSegment6(CitCommonUtil.getValue(rowData.getCell(10)).toString());
            trialBalance.setSegment7(CitCommonUtil.getValue(rowData.getCell(11)).toString());
            trialBalance.setSegment8(CitCommonUtil.getValue(rowData.getCell(12)).toString());
            trialBalance.setSegment9(CitCommonUtil.getValue(rowData.getCell(13)).toString());
            trialBalance.setSegment10(CitCommonUtil.getValue(rowData.getCell(14)).toString());
            trialBalance.setSegment1Name(CitCommonUtil.getValue(rowData.getCell(15)).toString());
            trialBalance.setSegment2Name(CitCommonUtil.getValue(rowData.getCell(16)).toString());

            trialBalance.setAccountDescription(CitCommonUtil.getValue(rowData.getCell(17)).toString());
            trialBalance.setSegment4Name(CitCommonUtil.getValue(rowData.getCell(18)).toString());
            trialBalance.setSegment5Name(CitCommonUtil.getValue(rowData.getCell(19)).toString());
            trialBalance.setSegment6Name(CitCommonUtil.getValue(rowData.getCell(20)).toString());
            trialBalance.setSegment7Name(CitCommonUtil.getValue(rowData.getCell(21)).toString());
            trialBalance.setSegment8Name(CitCommonUtil.getValue(rowData.getCell(22)).toString());
            trialBalance.setSegment9Name(CitCommonUtil.getValue(rowData.getCell(23)).toString());
            trialBalance.setSegment10Name(CitCommonUtil.getValue(rowData.getCell(24)).toString());
            //此期间默认为12
            trialBalance.setAccountPeriod(12);
            trialBalance.setBeginningBalance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(25)).toString()));

            trialBalance.setJan(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(26)).toString()));
            trialBalance.setFeb(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(27)).toString()));
            trialBalance.setMar(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(28)).toString()));
            trialBalance.setApr(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(29)).toString()));
            trialBalance.setMay(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(30)).toString()));
            trialBalance.setJun(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(31)).toString()));
            trialBalance.setJul(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(32)).toString()));
            trialBalance.setAug(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(33)).toString()));
            trialBalance.setSep(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(34)).toString()));
            trialBalance.setOct(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(35)).toString()));
            trialBalance.setNov(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(36)).toString()));
            trialBalance.setDecember(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(37)).toString()));
            trialBalance.setThirteen(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(38)).toString()));

            trialBalance.setEndingBalance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(39)).toString()));

            trialBalanceList.add(trialBalance);
        }
        //判断导入的方式
        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType) && projectIdList.size()>0) {
            //覆盖导入要删除原有的该操作人在当前期间导入的数据，且更改之前导入的日志记录状态
            msg = "覆盖导入成功";
            CitTrialBalanceExample citTrialBalanceExample = new CitTrialBalanceExample();
            CitTrialBalanceExample.Criteria criteria = citTrialBalanceExample.createCriteria();
            criteria.andPeriodEqualTo(period);
            criteria.andProjectIdIn(projectIdList);
            citTrialBalanceMapper.deleteByExample(citTrialBalanceExample);
            updateImportLog(EnumCitImportType.TrialBalance.getCode());
        }
        int insertBatchNum = 0;
        if(trialBalanceList != null && trialBalanceList.size()>0){
            insertBatchNum = citTrialBalanceMapper.insertBatch(trialBalanceList);
        }

        //循环遍历成功导入的数据，组装日志记录实体并放入集合
        for (Map.Entry<String, Integer> entry : companySuccessMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLog = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.TrialBalance.getCode(), period,
                    split[3], "试算平衡表", true);
            citDataImportLog.setRecordSize(entry.getValue());
            citDataImportLog.setErrorMsg(msg);
            citDataImportLogList.add(citDataImportLog);
        }
        for (Map.Entry<String, Integer> entry : companyFailMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.TrialBalance.getCode(), period,
                    split[3], "试算平衡表", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            citDataImportLogList.add(citDataImportLogError);
        }
        for (Map.Entry<String, Integer> entry : noCompanyMap.entrySet()) {
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(entry.getKey(), "", "",
                    EnumCitImportType.TrialBalance.getCode(), period,
                    "", "试算平衡表", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.NoCompanyError);
            citDataImportLogList.add(citDataImportLogError);
        }
        for (Map.Entry<String, Integer> entry : noProjectMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.TrialBalance.getCode(), period,
                    "", "试算平衡表", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.NoProjectError);
            citDataImportLogList.add(citDataImportLogError);
        }

        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入底稿&科目Mapping表
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importDraftAccountMapping(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citDraftAccountMapping");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitDraftAccountMapping> citDAMappings = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 1; rowNum <= sheet.getLastRowNum(); rowNum++) {

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.DraftAccountMapping.getCode(), period,
                    "", "底稿&科目Mapping表", false);
            citDataImportLogError.setRecordSize(1);
            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDraftAccountMapping citDAMapping = new CitDraftAccountMapping();
            citDAMapping.setId(idService.nextId());
            citDAMapping.setPeriod(period);
            citDAMapping.setCreatedBy(currentUserName);

            citDAMapping.setDraftCode(CitCommonUtil.getValue(rowData.getCell(0)).toString());
            citDAMapping.setDraftName(CitCommonUtil.getValue(rowData.getCell(1)).toString());
            citDAMapping.setLargeAttribute(CitCommonUtil.getValue(rowData.getCell(2)).toString());

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(3));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行小类为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citDAMapping.setAttribute(cellValue.toString());

            citDAMapping.setWpPosition(CitCommonUtil.getValue(rowData.getCell(4)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(5));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行科目代码为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            DecimalFormat df=new DecimalFormat("0");
            citDAMapping.setAcctCode(cellValue.toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(6));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行科目名称为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citDAMapping.setAccountName(cellValue.toString());

            citDAMapping.setPlRowSet(CitCommonUtil.getValue(rowData.getCell(7)).toString());
            citDAMapping.setBsRowSet(CitCommonUtil.getValue(rowData.getCell(8)).toString());
            citDAMapping.setFetchNumber(CitCommonUtil.getValue(rowData.getCell(9)).toString());
            citDAMapping.setRemark(CitCommonUtil.getValue(rowData.getCell(10)).toString());


            citDAMappings.add(citDAMapping);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitDraftAccountMappingExample example = new CitDraftAccountMappingExample();
            CitDraftAccountMappingExample.Criteria criteria = example.createCriteria();
            criteria.andPeriodEqualTo(period);
            citDAMappingMapper.deleteByExample(example);
            updateImportLog(EnumCitImportType.DraftAccountMapping.getCode());
        }

        int insertBatchNum = citDAMappingMapper.insertBatch(citDAMappings);
        CitDataImportLog citDataImportLog = generalCitDataImportLog("", "", "",
                EnumCitImportType.DraftAccountMapping.getCode(), period,
                "", "底稿&科目Mapping表", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入资产负债表（单家PRC）
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importBalanceSheetPrcAdjust(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitBalanceSheetPrcAdjust> citBSPrcAdjustList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
//        String companyName = CitCommonUtil.getValue(sheet.getRow(5).getCell(1)).toString();
//        String companyCode = CitCommonUtil.getValue(sheet.getRow(5).getCell(1)).toString();
//        if("".equals(companyCode)){
//            saveResult.setResult(false);
//            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
//            return saveResult;
//        }
//        List<Organization> organizations = getOrganizationByCode(companyCode).stream().filter(r->r.getCode()!=null&&!("".equals(r.getCode()))).collect(Collectors.toList());

        String companyCode = CitCommonUtil.getValue(sheet.getRow(5).getCell(1)).toString();
        if("".equals(companyCode)){
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }
        companyCode = companyCode.contains(".")?companyCode.substring(0,companyCode.indexOf(".")):companyCode;
        List<Organization> organizations = getOrganizationByCode(companyCode);
        String companyName = "";
        String orgId = "";
        Project project = null;
        String taxPayerId = "";
        if (organizations != null && organizations.size() > 0) {
            companyName = organizations.get(0).getName();
            orgId = organizations.get(0).getId();
            project = getProject(orgId, period);

            if(!orgList.contains(orgId)){
                saveResult.setResult(false);
                saveResult.setData(false);
                saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
                return saveResult;
            }

            if(project == null){
                saveResult.setResult(false);
                saveResult.setData(false);
                saveResult.setResultMsg(ErrorMessageCN.NoProjectError);
                return saveResult;
            }

            taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                    organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();

        }else{
            saveResult.setResult(false);
            saveResult.setData(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }
        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 10; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.BalanceSheetPrcAdjust.getCode(), period,
                    "", "资产负债表(单家PRC)",  false);
            citDataImportLogError.setRecordSize(1);

            CitBalanceSheetPrcAdjust citBSPrcAdjust = new CitBalanceSheetPrcAdjust();
            citBSPrcAdjust.setId(idService.nextId());
            citBSPrcAdjust.setPeriod(period);
            citBSPrcAdjust.setOrganizationId(orgId);
            citBSPrcAdjust.setProjectId(project.getId());
            citBSPrcAdjust.setCreateBy(currentUserName);

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(0));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行项目名称为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citBSPrcAdjust.setItemName(cellValue.toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(1));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行期初余额为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citBSPrcAdjust.setBegBal(new BigDecimal(cellValue.toString().replace(",","")));


            cellValue = CitCommonUtil.getValue(rowData.getCell(2));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行期末余额为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citBSPrcAdjust.setEndBal(new BigDecimal(cellValue.toString().replace(",","")));
            citBSPrcAdjustList.add(citBSPrcAdjust);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitBalanceSheetPrcAdjustExample example = new CitBalanceSheetPrcAdjustExample();
            CitBalanceSheetPrcAdjustExample.Criteria criteria = example.createCriteria();
            criteria.andCreateByEqualTo(currentUserName);
            criteria.andPeriodEqualTo(period);
            criteria.andProjectIdEqualTo(project.getId());
            citBSPrcAdjustMapper.deleteByExample(example);
            updateImportLog(EnumCitImportType.BalanceSheetPrcAdjust.getCode());
        }

        int insertBatchNum = citBSPrcAdjustMapper.insertBatch(citBSPrcAdjustList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                EnumCitImportType.BalanceSheetPrcAdjust.getCode(), period,
                companyName, "资产负债表(单家PRC)", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入利润表（单家PRC）
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importProfitPrcAdjust(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitProfitPrcAdjust> citProfitPrcAdjustList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
//        String companyName = CitCommonUtil.getValue(sheet.getRow(4).getCell(1)).toString();
        String companyCode = CitCommonUtil.getValue(sheet.getRow(5).getCell(1)).toString();
        if("".equals(companyCode)){
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }
        companyCode = companyCode.contains(".")?companyCode.substring(0,companyCode.indexOf(".")):companyCode;
        List<Organization> organizations = getOrganizationByCode(companyCode);
        String companyName = "";
        String orgId = "";
        Project project = null;
        String taxPayerId = "";
        if (organizations != null && organizations.size() > 0) {
            orgId = organizations.get(0).getId();
            companyName = organizations.get(0).getName();

            if(!orgList.contains(orgId)){
                saveResult.setResult(false);
                saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
                return saveResult;
            }
            taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                    organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
            project = getProject(orgId, period);

            if(project == null){
                saveResult.setResult(false);
                saveResult.setData(false);
                saveResult.setResultMsg(ErrorMessageCN.NoProjectError);
                return saveResult;
            }

        }else{
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
            return saveResult;
        }
        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 10; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                    EnumCitImportType.ProfitPrcAdjust.getCode(), period,
                    companyName, "利润表(单家PRC)",  false);
            citDataImportLogError.setRecordSize(1);

            CitProfitPrcAdjust citProfitPrcAdjust = new CitProfitPrcAdjust();
            citProfitPrcAdjust.setId(idService.nextId());
            citProfitPrcAdjust.setOrganizationId(orgId);
            citProfitPrcAdjust.setProjectId(project.getId());
            citProfitPrcAdjust.setPeriod(period);
            citProfitPrcAdjust.setCreateBy(currentUserName);

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(0));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行项目名称为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citProfitPrcAdjust.setItemName(CitCommonUtil.getValue(rowData.getCell(0)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(1));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行当期发生为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citProfitPrcAdjust.setPeriodAmt(new BigDecimal(cellValue.toString().replace(",","")));

            cellValue = CitCommonUtil.getValue(rowData.getCell(2));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行本年累计为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citProfitPrcAdjust.setYtdAmt(new BigDecimal(cellValue.toString().replace(",","")));
            citProfitPrcAdjustList.add(citProfitPrcAdjust);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitProfitPrcAdjustExample example = new CitProfitPrcAdjustExample();
            CitProfitPrcAdjustExample.Criteria criteria = example.createCriteria();
            criteria.andCreateByEqualTo(currentUserName);
            criteria.andPeriodEqualTo(period);
            criteria.andProjectIdEqualTo(project.getId());
            citProfitPrcAdjustMapper.deleteByExample(example);
            updateImportLog(EnumCitImportType.ProfitPrcAdjust.getCode());
        }

        int insertBatchNum = citProfitPrcAdjustMapper.insertBatch(citProfitPrcAdjustList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                EnumCitImportType.ProfitPrcAdjust.getCode(), period,
                companyName, "利润表(单家PRC)", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入EAM资产处置金额记录表
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importEAMAssetsDisposal(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitEAMAssetsDisposal> citEAMAssetsList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 3; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.EAMAssetsDisposal.getCode(), period,
                    "", "EAM资产处置金额记录表",  false);
            citDataImportLogError.setRecordSize(1);

            CitEAMAssetsDisposal citEAMAssetsDisposal = new CitEAMAssetsDisposal();
            citEAMAssetsDisposal.setId(idService.nextId());
            citEAMAssetsDisposal.setPeriod(period);
            citEAMAssetsDisposal.setCreatedBy(currentUserName);

            String[] split = CitCommonUtil.getValue(rowData.getCell(1)).toString().split("-");
            StringBuilder sb = new StringBuilder(split[0]);
//            if(new Integer(split[1]) + 1 < 10){
//                sb.append(0);
//            }
            sb.append(split[1]);
            citEAMAssetsDisposal.setOccurPeriod(new Integer(sb.toString()));

            citEAMAssetsDisposal.setOaAssetBillType(CitCommonUtil.getValue(rowData.getCell(2)).toString());
            citEAMAssetsDisposal.setOaAssetNumber(CitCommonUtil.getValue(rowData.getCell(3)).toString());
            citEAMAssetsDisposal.setScrapReason(CitCommonUtil.getValue(rowData.getCell(4)).toString());
            citEAMAssetsDisposal.setAssetNumber(CitCommonUtil.getValue(rowData.getCell(5)).toString());

            citEAMAssetsDisposal.setAssetLabelNumber(CitCommonUtil.getValue(rowData.getCell(6)).toString());
            citEAMAssetsDisposal.setCompensationSaleAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(7)).toString()));

            citEAMAssetsDisposal.setLiableEmployeeNum(CitCommonUtil.getValue(rowData.getCell(8)).toString());
            citEAMAssetsDisposal.setLiableEmployeeName(CitCommonUtil.getValue(rowData.getCell(9)).toString());
            citEAMAssetsDisposal.setRemitEmployeeNum(CitCommonUtil.getValue(rowData.getCell(10)).toString());
            citEAMAssetsDisposal.setRemitEmployeeName(CitCommonUtil.getValue(rowData.getCell(11)).toString());
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Object cellValue = CitCommonUtil.getValue(rowData.getCell(12));
            if(!"".equals(cellValue)){
                citEAMAssetsDisposal.setRemitTime(sdf.parse(cellValue.toString()));
            }
            cellValue = CitCommonUtil.getValue(rowData.getCell(13));
            if("".equals(cellValue)){
                cellValue = "0";
            }
            citEAMAssetsDisposal.setRemitAmount(new BigDecimal(cellValue.toString()));
            citEAMAssetsDisposal.setGatheringCompanyName(CitCommonUtil.getValue(rowData.getCell(14)).toString());
            citEAMAssetsDisposal.setGatheringCompanyAccount(CitCommonUtil.getValue(rowData.getCell(15)).toString());
            citEAMAssetsDisposal.setRemark(CitCommonUtil.getValue(rowData.getCell(16)).toString());

            citEAMAssetsList.add(citEAMAssetsDisposal);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitEAMAssetsDisposalExample example = new CitEAMAssetsDisposalExample();
            CitEAMAssetsDisposalExample.Criteria criteria = example.createCriteria();
            criteria.andCreatedByEqualTo(currentUserName);
            criteria.andPeriodEqualTo(period);
            citEAMAssetsMapper.deleteByExample(example);
            updateImportLog(EnumCitImportType.EAMAssetsDisposal.getCode());
        }

        int insertBatchNum = citEAMAssetsMapper.insertBatch(citEAMAssetsList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog("", "", "",
                EnumCitImportType.EAMAssetsDisposal.getCode(), period,
                "", "EAM资产处置金额记录表", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入预提重分类数据
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importSalaryAdvance(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer month, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitSalaryAdvance> citSalaryAdvanceList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();

        Map<String, Integer> companySuccessMap = new HashMap();
        Map<String, Integer> companyFailMap = new HashMap();
        Map<String, Integer> noCompanyMap = new HashMap();
        Map<String, Integer> noProjectMap = new HashMap();
        String currentUserName = authUserHelper.getCurrentAuditor().get();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 3; rowNum <= sheet.getLastRowNum(); rowNum++) {

            DecimalFormat df=new DecimalFormat("0");
            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.SalaryAdvance.getCode(), period,
                    "", "预提重分类数据源", false);
            citDataImportLogError.setRecordSize(1);

            Object cellValueCode = CitCommonUtil.getValue(rowData.getCell(5));
            if("".equals(cellValueCode)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行PO主体CODE为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            String companyCode = cellValueCode.toString().replace(".0","");
            //根据code（主体）获取机构相关信息
            List<Organization> organizations = getOrganizationByCode(companyCode);
            String companyName = "";
            String orgId = "";
            Project project = null;
            String taxPayerId = "";
            //判断是否查询到该机构，若数据库中不存在则不执行导入并记录日志
            if (organizations != null && organizations.size() > 0) {

                orgId = organizations.get(0).getId();
                taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                        organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                companyName = organizations.get(0).getName();

                //拼接公司相关信息，作为键，其Excel出现的数量作为值
                StringBuilder sb = new StringBuilder(companyCode);
                sb.append(",");
                sb.append(orgId);
                sb.append(",");
                sb.append(taxPayerId);
                sb.append(",");
                sb.append(companyName);
                if(!orgList.contains(orgId)){
                    companyFailMap.put(sb.toString(),companyFailMap.containsKey(sb.toString())?companyFailMap.get(sb.toString())+1:1);
                    continue;
                }

                project = getProject(orgId, period);
                if(project == null){
                    noProjectMap.put(sb.toString(),noProjectMap.containsKey(sb.toString())?noProjectMap.get(sb.toString())+1:1);
                    continue;
                }
                companySuccessMap.put(sb.toString(),companySuccessMap.containsKey(sb.toString())?companySuccessMap.get(sb.toString())+1:1);

            }else{
                noCompanyMap.put(companyCode,noCompanyMap.containsKey(companyCode)?noCompanyMap.get(companyCode)+1:1);
                continue;
            }

            CitSalaryAdvance citSalaryAdvance = new CitSalaryAdvance();
            citSalaryAdvance.setId(idService.nextId());
            citSalaryAdvance.setOrganizationId(orgId);
            citSalaryAdvance.setProjectId(project.getId());
            citSalaryAdvance.setPeriod(period);
            citSalaryAdvance.setMonth(month);
            citSalaryAdvance.setCreatedBy(currentUserName);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(0));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行PO号为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            //当单元格是常规格式时会有小数点，需去掉小数点
            citSalaryAdvance.setPoNo(cellValue.toString().replace(".0",""));
            cellValue = CitCommonUtil.getValue(rowData.getCell(1));
            citSalaryAdvance.setPoRow(cellValue.toString().replace(".0",""));
            cellValue = CitCommonUtil.getValue(rowData.getCell(2));
            if(!"".equals(cellValue)){
                citSalaryAdvance.setPoCreateTime(sdf.parse(cellValue.toString()));
            }

            citSalaryAdvance.setLadingBillPerson(CitCommonUtil.getValue(rowData.getCell(3)).toString());
            citSalaryAdvance.setPurchasePerson(CitCommonUtil.getValue(rowData.getCell(4)).toString());
            citSalaryAdvance.setPoSubjectCode(companyCode);

            cellValue = CitCommonUtil.getValue(rowData.getCell(6));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行PO主体为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citSalaryAdvance.setPoSubjectName(cellValue.toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(7));
            citSalaryAdvance.setCostCenterCode(cellValue.toString().replace(".0",""));
            citSalaryAdvance.setCostCenter(CitCommonUtil.getValue(rowData.getCell(8)).toString());
            citSalaryAdvance.setSupplier(CitCommonUtil.getValue(rowData.getCell(9)).toString());
            citSalaryAdvance.setItem(CitCommonUtil.getValue(rowData.getCell(10)).toString());
            citSalaryAdvance.setStandardMoney(CitCommonUtil.getValue(rowData.getCell(11)).toString());
            citSalaryAdvance.setCurrency(CitCommonUtil.getValue(rowData.getCell(12)).toString());
            cellValue = CitCommonUtil.getValue(rowData.getCell(13));
            if(!"".equals(cellValue)){
                citSalaryAdvance.setOrderAmount(new BigDecimal(cellValue.toString()));
            }
            citSalaryAdvance.setExecutedServiceSchedule(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(14)).toString()));
            citSalaryAdvance.setCoupaTotalAccept(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(15)).toString()));

            cellValue = CitCommonUtil.getValue(rowData.getCell(16));
            if("".equals(cellValue)){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行预提为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citSalaryAdvance.setAdvance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(16)).toString()));
            citSalaryAdvance.setUnexecutedServiceSchedule(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(17)).toString()));
            citSalaryAdvance.setApprovedStandardInvoiceAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(18)).toString()));
            citSalaryAdvance.setPaidInvoiceAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(19)).toString()));
            citSalaryAdvance.setUsableInvoiceAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(20)).toString()));
            citSalaryAdvance.setUsableBalance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(21)).toString()));
            citSalaryAdvance.setAdvanceWriteOff(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(22)).toString()));


            citSalaryAdvance.setSubjectCode(CitCommonUtil.getValue(rowData.getCell(27)).toString());
            citSalaryAdvance.setSubjectDescription(CitCommonUtil.getValue(rowData.getCell(28)).toString());
            citSalaryAdvance.setAuxiliarySubject(CitCommonUtil.getValue(rowData.getCell(29)).toString());
            citSalaryAdvance.setAuxiliarySubjectDescription(CitCommonUtil.getValue(rowData.getCell(30)).toString());
            citSalaryAdvance.setProfitCenter(CitCommonUtil.getValue(rowData.getCell(31)).toString());
            citSalaryAdvance.setProfitCenterDescription(CitCommonUtil.getValue(rowData.getCell(32)).toString());
            citSalaryAdvance.setProduct(CitCommonUtil.getValue(rowData.getCell(33)).toString());
            citSalaryAdvance.setProductDescription(CitCommonUtil.getValue(rowData.getCell(34)).toString());
            //项目，不是卡片
            citSalaryAdvance.setProject(CitCommonUtil.getValue(rowData.getCell(35)).toString());
            citSalaryAdvance.setProjectDescription(CitCommonUtil.getValue(rowData.getCell(36)).toString());
            citSalaryAdvance.setCompany(CitCommonUtil.getValue(rowData.getCell(37)).toString());
            citSalaryAdvance.setCompanyDescription(CitCommonUtil.getValue(rowData.getCell(38)).toString());
            citSalaryAdvance.setSegment1(CitCommonUtil.getValue(rowData.getCell(39)).toString());
            citSalaryAdvance.setSegment1Description(CitCommonUtil.getValue(rowData.getCell(40)).toString());
            citSalaryAdvance.setSegment2(CitCommonUtil.getValue(rowData.getCell(41)).toString());
            citSalaryAdvance.setSegment2Description(CitCommonUtil.getValue(rowData.getCell(42)).toString());

            citSalaryAdvanceList.add(citSalaryAdvance);
        }

        //判断导入的方式
        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            //覆盖导入要删除原有的该操作人在当前期间导入的数据，且更改之前导入的日志记录状态
            msg = "覆盖导入成功";
            CitSalaryAdvanceExample example = new CitSalaryAdvanceExample();
            CitSalaryAdvanceExample.Criteria criteria = example.createCriteria();
            criteria.andCreatedByEqualTo(currentUserName);
            criteria.andPeriodEqualTo(period);
            criteria.andMonthEqualTo(month);
            citSalaryAdvanceMapper.deleteByExample(example);
            updateImportLog(EnumCitImportType.SalaryAdvance.getCode());
        }
        int insertBatchNum = 0;
        if(citSalaryAdvanceList.size()>0){
            insertBatchNum = citSalaryAdvanceMapper.insertBatch(citSalaryAdvanceList);
        }

        //循环遍历成功导入的数据，组装日志记录实体并放入集合
        for (Map.Entry<String, Integer> entry : companySuccessMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLog = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.SalaryAdvance.getCode(), period,
                    split[3], "预提重分类数据源",  true);
            citDataImportLog.setRecordSize(entry.getValue());
            citDataImportLog.setErrorMsg(msg);
            citDataImportLogList.add(citDataImportLog);
        }
        for (Map.Entry<String, Integer> entry : companyFailMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.SalaryAdvance.getCode(), period,
                    split[3], "预提重分类数据源",  false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            citDataImportLogList.add(citDataImportLogError);
        }
        for (Map.Entry<String, Integer> entry : noCompanyMap.entrySet()) {
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(entry.getKey(), "", "",
                    EnumCitImportType.SalaryAdvance.getCode(), period,
                    "", "预提重分类数据源", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.NoCompanyError);
            citDataImportLogList.add(citDataImportLogError);
        }
        for (Map.Entry<String, Integer> entry : noProjectMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.SalaryAdvance.getCode(), period,
                    "", "预提重分类数据源", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.NoProjectError);
            citDataImportLogList.add(citDataImportLogError);
        }
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setData(insertBatchNum);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 获取导入的日记账数据
     *
     * @return
     */
    public List<CitJournalEntryAdjust> getJournalAdjust() {
        CitJournalEntryAdjustExample example = new CitJournalEntryAdjustExample();
        CitJournalEntryAdjustExample.Criteria criteria = example.createCriteria();
        List<CitJournalEntryAdjust> citJournalEntryAdjusts = citJournalEntryAdjustMapper.selectByExample(example);
        return citJournalEntryAdjusts;
    }

    /**
     * 获取导入的试算平衡表数据
     *
     * @return
     */
    public List<CitJournalEntryAdjust> getTrialBalance() {
        CitJournalEntryAdjustExample example = new CitJournalEntryAdjustExample();
        CitJournalEntryAdjustExample.Criteria criteria = example.createCriteria();
        List<CitJournalEntryAdjust> citJournalEntryAdjusts = citJournalEntryAdjustMapper.selectByExample(example);
        return citJournalEntryAdjusts;
    }


    private BigDecimal add(BigDecimal v1, BigDecimal v2) {
        v1 = null == v1 ? BigDecimal.ZERO : v1;
        v2 = null == v2 ? BigDecimal.ZERO : v2;
        return v1.add(v2);
    }

    private CitDataImportLog generalCitDataImportLog(String companyCode, String orgId, String taxPayerId, Integer type, Integer period, String companyName, String fileType, boolean importResult) {
        CitDataImportLog citDataImportLog = new CitDataImportLog();
        citDataImportLog.setId(idService.nextId());
        citDataImportLog.setTaxpayerIdNum(taxPayerId);
        citDataImportLog.setType(type);
        citDataImportLog.setOrganizationId(orgId);
        citDataImportLog.setCompanyCode(companyCode);
        citDataImportLog.setCompanyName(companyName);
        citDataImportLog.setFileType(fileType);
        citDataImportLog.setPeriod(period);
        citDataImportLog.setPeriodStatus("人工导入");
        citDataImportLog.setOperator(authUserHelper.getCurrentAuditor().get());
        citDataImportLog.setImportResult(importResult);
        return citDataImportLog;
    }

    /**
     * 根据文件类型修改版本状态，置为历史版本
     * @param type
     * @return
     */
    private int updateImportLog(Integer type) {
        int updateDisplayOld = citDataImportLogMapper.updateDisplayOld(type,authUserHelper.getCurrentAuditor().get());
        return updateDisplayOld;
    }

    private int addDataImportLog(List<CitDataImportLog> dataImportLogs) {
        //在添加import log前先将原来导入的数据版本置为旧版本
////        citDataImportLogMapper.updateDisplayOld(dataImportLogs.get(0).getType());
        int res = 0;
        for (CitDataImportLog dataImportLog : dataImportLogs) {
            res += citDataImportLogMapper.insertSelective(dataImportLog);
        }
        return res;
    }

    /**
     * 根据导入类型获取导入记录
     *
     * @param type
     * @return
     */
    public List<CitDataImportLogDto> getCitDataImportLog(Integer type) {
        List<CitDataImportLogDto> citDataImportLogDtos = Lists.newArrayList();
        CitDataImportLogExample citDataImportLogExample = new CitDataImportLogExample();
        //判断传入的参数是否可用
        if (EnumImportType.Undefined.getCode().equals(type)) {
            citDataImportLogExample.setOrderByClause("update_time desc");
        } else {
            citDataImportLogExample.createCriteria().andTypeEqualTo(type).andDisplayEqualTo(true).andOperatorEqualTo(authUserHelper.getCurrentAuditor().get());
            citDataImportLogExample.setOrderByClause("update_time desc");
        }
        //把从数据库获取的实体转化成页面Dto
        citDataImportLogMapper.selectByExample(citDataImportLogExample).forEach(citDataImportLog -> {
            CitDataImportLogDto dataImportLogDto = new CitDataImportLogDto();
            beanUtil.copyProperties(citDataImportLog, dataImportLogDto);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            dataImportLogDto.setCreateTimeStr(sdf.format(citDataImportLog.getCreateTime()));
            citDataImportLogDtos.add(dataImportLogDto);
        });
        return citDataImportLogDtos;
    }

    /**
     * 根据公司code获取机构相关信息
     * @param companyCode
     * @return
     */
    private List<Organization> getOrganizationByCode(String companyCode) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andCodeEqualTo(companyCode);
        List<Organization> organizations = organizationMapper.selectByExample(example);
        return organizations;
    }
    /**
     * 根据公司code获取机构相关信息
     * @param companyName
     * @return
     */
    private List<Organization> getOrganizationByName(String companyName) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andNameEqualTo(companyName);
        List<Organization> organizations = organizationMapper.selectByExample(example);
        return organizations;
    }

    /**
     * 根据机构ID和期间查询出project（即卡片）
     * @param orgId
     * @param periodYear
     * @return
     */
    private Project getProject(String orgId, Integer periodYear) {
        ProjectExample pExample = new ProjectExample();
        pExample.createCriteria().andOrganizationIdEqualTo(orgId).andYearEqualTo(periodYear);
        List<Project> pList = projectMapper.selectByExample(pExample);
        if(pList != null && !pList.isEmpty()){
            return pList.get(0);
        }else{
            return null;
        }
    }

    public void generateTb(String projectId){
        Project project = projectMapper.selectByPrimaryKey(projectId);
        //删除自动生成的试算平衡表相关数据
        CitTrialBalanceExample citTbExample = new CitTrialBalanceExample();
        citTbExample.createCriteria().andPeriodEqualTo(project.getYear()).andProjectIdEqualTo(projectId);
        citTrialBalanceMapper.deleteByExample(citTbExample);
        logger.info("删除符合条件的试算平衡表数据");
        CitTbamExample citTbamExample = new CitTbamExample();
        citTbamExample.createCriteria().andPeriodEqualTo(project.getYear()).andProjectIdEqualTo(projectId);
        logger.info("删除符合条件的试算平衡表Mapping数据");
        citTbamMapper.deleteByExample(citTbamExample);
        updateImportLog(EnumCitImportType.JournalAdjust.getCode());
        List<String> orgList = getOrgList(project);
        autoGeneTB(project, orgList, authUserHelper.getCurrentAuditor().get());
    }

    /**
     * 获取该机构以及该机构的子机构的所有机构主键
     * @param project
     * @return
     */
    public List<String> getOrgList(Project project){
        //该机构以及该机构的子机构的所有机构主键集合变量
        List<String> orgListTemp = new ArrayList<>();
        Organization organization = organizationMapper.selectByPrimaryKey(project.getOrganizationId());
        orgListTemp.add(project.getOrganizationId());
        if(StringUtils.isBlank(organization.getParentId())){
            Integer year = project.getYear();
            String orgId = project.getOrganizationId();
            //要处理日记账的卡片主键(projectId)集合变量
            List<Project> projects = projectMapper.queryChildOrgAndProjectByOrgId(year, orgId);
            for (Project pro : projects) {
                orgListTemp.add(pro.getOrganizationId());
            }
            String currentUserName = authUserHelper.getCurrentAuditor().get();
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try{
                        for (Project pro : projects) {
                            List<String> orgChildTemp = new ArrayList<>();
                            orgChildTemp.add(pro.getOrganizationId());
                            autoGeneTB(pro,orgChildTemp,currentUserName);
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                    }
                }
            }).start();
        }
        return orgListTemp;
    }

    /**
     * 自动生成试算平衡表和join出Mapping版的attribute mapping TB
     * @param project
     * @param orgList
     * @return
     */
    public void autoGeneTB(Project project, List<String> orgList, String currentUserName){

        //第一步，根据合并日记账计算出借方发生额，贷方发生额，科目代码等相关信息
        StringBuilder periodSb = new StringBuilder();
        periodSb.append(project.getYear());
        periodSb.append("%");
        //第二步，查出cit_journal_entry_adjust该期间的调整日记账，
        List<CitJournalEntryAdjust> citJournalEntryAdjustList = citJournalEntryAdjustMapper.sumAmountByCondition(periodSb.toString(), orgList);

        List<CitTrialBalance> trialBalanceList = new ArrayList<>();
        //存放TB与Mapping结合的实体
        List<CitTbam> citTbamList = new ArrayList<>();
        for (CitJournalEntryAdjust citJournal:citJournalEntryAdjustList) {

            CitTrialBalance trialBalance = new CitTrialBalance();

            trialBalance.setPeriod(project.getYear());
            trialBalance.setCreateBy(currentUserName);
            trialBalance.setOrganizationId(project.getOrganizationId());
            trialBalance.setProjectId(project.getId());
            trialBalance.setAccountCode(citJournal.getSubjectCode());
            trialBalance.setAccountDescription(citJournal.getSubjectName());
            //此期间默认为12
            trialBalance.setAccountPeriod(12);
            //借方发生额
            trialBalance.setDebitAmount(citJournal.getAccountedDr());
            //贷方发生额
            trialBalance.setCreditAmount(citJournal.getAccountedCr());

            //查询出该科目代码的上一年的期末余额，查询条件为科目代码和期间
            BigDecimal lastYearBegBla = new BigDecimal(0);
            CitTrialBalanceExample citTrialBalanceExample = new CitTrialBalanceExample();
            CitTrialBalanceExample.Criteria criteria = citTrialBalanceExample.createCriteria();
            criteria.andOrganizationIdEqualTo(project.getOrganizationId());
            criteria.andAccountCodeEqualTo(citJournal.getSubjectCode());
            criteria.andPeriodEqualTo(project.getYear()-1);
            List<CitTrialBalance> trialBalances = citTrialBalanceMapper.selectByExample(citTrialBalanceExample);
            if(trialBalances != null && trialBalances.size() > 0){
                lastYearBegBla = lastYearBegBla.add(trialBalances.get(0).getBeginningBalance());
            }
            trialBalance.setBeginningBalance(lastYearBegBla);

            //计算出期末余额，期末余额为期初余额+借方发生额-贷方发生额
            trialBalance.setEndingBalance(lastYearBegBla.add(citJournal.getAccountedDr()).subtract(citJournal.getAccountedCr()));

            //根据科目代码及期间查询出相应底稿&科目Mapping数据并赋值给实体CitTBam
            CitDraftAccountMappingExample citDAMappingExample = new CitDraftAccountMappingExample();
            citDAMappingExample.createCriteria().andAcctCodeEqualTo(citJournal.getSubjectCode()).andPeriodEqualTo(project.getYear());
            List<CitDraftAccountMapping> citDraftAccountMappings = citDAMappingMapper.selectByExample(citDAMappingExample);
            for (CitDraftAccountMapping daMapping : citDraftAccountMappings) {
                CitTbam citTbam = new CitTbam();
                //把根据日记账算出的TB数据复制到Mapping的TB
                beanUtil.copyProperties(trialBalance,citTbam);
                citTbam.setId(idService.nextId());
                citTbam.setAttribute(daMapping.getAttribute());
                citTbam.setAccountDescription(citJournal.getSubjectName());
                citTbamList.add(citTbam);
            }
//            citDraftAccountMappings.stream().forEach(daMapping -> {
//                citTbam.setId(idService.nextId());
//                citTbam.setAttribute(daMapping.getAttribute());
//                citTbamList.add(citTbam);
//                if("70019900".equals(citTbam.getAccountCode())){
//                    System.out.println(citTbam.getId());
//                }
//            });
            trialBalance.setId(idService.nextId());
            trialBalanceList.add(trialBalance);
        }
        if(trialBalanceList != null && trialBalanceList.size() >0){
            citTrialBalanceMapper.insertBatch(trialBalanceList);
        }
        if(citTbamList != null && citTbamList.size() >0){
            citTbamMapper.insertBatch(citTbamList);
        }
        logger.info("自动生成TB成功");
    }

}
