package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.constant.enums.RevenueConfEnum;
import pwc.taxtech.atms.dpo.OrgSelectDto;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.revenuconf.RevConfAddDto;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfParam;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfResult;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.RevenueConfigMapper;
import pwc.taxtech.atms.vat.entity.RevenueConfig;
import pwc.taxtech.atms.vat.entity.RevenueConfigExample;

import javax.annotation.Resource;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class RevenueConfService extends BaseService {

    @Resource
    private RevenueConfigMapper revenueConfigMapper;
    @Resource
    private OrganizationServiceImpl organizationService;

    /**
     * 分页查询可查看的配置信息
     *
     * @param param RevenueConfParam
     * @return PageInfo
     */
    public PageInfo<RevenueConfResult> queryPage(RevenueConfParam param) {
        List<OrgSelectDto> orgDtoList = organizationService.getMyOrgList();
        if (CollectionUtils.isEmpty(orgDtoList)) {
            return new PageInfo<>();
        }
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        RevenueConfigExample example = new RevenueConfigExample();
        example.createCriteria().andOrgIdIn(orgDtoList.stream().map(OrgSelectDto::getId).collect(Collectors.toList()));
        example.setOrderByClause("org_id");
        PageInfo<RevenueConfResult> pageInfo = new PageInfo<>(revenueConfigMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new RevenueConfResult())).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    public List<RevenueConfResult> queryAll(RevenueConfParam param) {
        RevenueConfigExample example = new RevenueConfigExample();
        example.createCriteria().andOrgIdEqualTo(param.getOrgId()).andStartDateLessThanOrEqualTo(param.getStartDate()).andEndDateGreaterThanOrEqualTo(param.getEndDate());
        return revenueConfigMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new RevenueConfResult())).collect(Collectors.toList());
    }

    /**
     * 新增配置
     *
     * @param config RevenueConfig
     */
    public void addConfig(RevenueConfig config) {
        //todo 重复校验
        config.setId(idService.nextId());
        revenueConfigMapper.insertSelective(parseEntity(config));
    }

    /**
     * 前台批量新增
     *
     * @param addDto RevConfAddDto
     */
    public ApiResultDto addConfig(RevConfAddDto addDto) {


        RevenueConfigExample revenueConfigExample = new RevenueConfigExample();
        revenueConfigExample.createCriteria().andNameEqualTo(addDto.getName());
        List<RevenueConfig> list = revenueConfigMapper.selectByExample(revenueConfigExample);
        if (list.size() > 0) {
            ApiResultDto apiResultDto = ApiResultDto.success();
            apiResultDto.setMessage("DuplicateName");
            return apiResultDto;
        }

        if (!CollectionUtils.isEmpty(addDto.getOrgList())) {
            addDto.getOrgList().forEach(id -> {
                addDto.setId(idService.nextId());
                addDto.setOrgId(id);
                revenueConfigMapper.insertSelective(parseEntity(addDto));
            });
            return ApiResultDto.success();
        }

        return ApiResultDto.fail();
    }

    /**
     * 更新配置
     *
     * @param config RevenueConfig
     */
    public void updateConfig(RevenueConfig config) {
        //todo 重复校验
        revenueConfigMapper.updateByPrimaryKeySelective(parseEntity(config));
    }

    public void delConfig(List<Long> idList) {
        if (!CollectionUtils.isEmpty(idList)) {
            idList.forEach(id -> revenueConfigMapper.deleteByPrimaryKey(id));
        }
    }

    private RevenueConfig parseEntity(RevenueConfig config) {
        if (config.getAccountType() != RevenueConfEnum.AccountType.Account.getCode().intValue()) {
            config.setTbSegment3(StringUtils.EMPTY);
            config.setTbSegment5(StringUtils.EMPTY);
            config.setTbSegment6(StringUtils.EMPTY);
        }
        if (config.getTaxBase() != RevenueConfEnum.TaxBase.Period_Cr.getCode().intValue()) {
            config.setBaseCrCode(StringUtils.EMPTY);
        }
        if (config.getTaxBase() != RevenueConfEnum.TaxBase.Period_Dr.getCode().intValue()) {
            config.setBaseDrCode(StringUtils.EMPTY);
        }
        return config;
    }

    public List<String> upload(MultipartFile file, Integer type) throws Exception {
        List<String> unSuccessList = new ArrayList<>();
        List<RevenueConfig> dataList = new ArrayList<>();
        InputStream inputStream = file.getInputStream();
        Workbook workbook = WorkbookFactory.create(inputStream);
        Sheet sheet = workbook.getSheetAt(0);
        List<OrgSelectDto> orgDtoList = organizationService.getMyOrgList();
        List<String> orgIds = new ArrayList<>();
        String unSuccessMsg;
        for (int r = sheet.getFirstRowNum()+1; r <= sheet.getLastRowNum(); r++) {
            Row row = sheet.getRow(r);
            unSuccessMsg = "第"+(r+1)+"行：";
            String orgName = POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_2.getIndex()));
            Optional<OrgSelectDto> optional = orgDtoList.stream().filter(o -> StringUtils.equals(o.getName(), orgName)).findFirst();
            if (!optional.isPresent()) {
                unSuccessMsg +="|公司不存在|";
                unSuccessList.add(unSuccessMsg);
                continue;
            }
            String msg = verifyColumn(row);
            if(StringUtils.isNotBlank(msg)){
                unSuccessMsg+=msg;
                unSuccessList.add(unSuccessMsg);
                continue;
            }
            RevenueConfig revenueConfig = new RevenueConfig();
            revenueConfig.setId(idService.nextId());
            revenueConfig.setSerialNo("");
            revenueConfig.setAccountName("");
            revenueConfig.setOrgId(optional.get().getId());
            orgIds.add(optional.get().getId());
            revenueConfig.setName(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_1.getIndex())));
            revenueConfig.setAccountType(RevenueConfEnum.AccountType.REVERSAL_MAPPING.get(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_3.getIndex()))));
            revenueConfig.setTbSegment3(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_4.getIndex())));
            if(StringUtils.isBlank(revenueConfig.getTbSegment3())){
                revenueConfig.setTbSegment3("");
            }
            revenueConfig.setTbSegment5(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_5.getIndex())));
            if(StringUtils.isBlank(revenueConfig.getTbSegment5())){
                revenueConfig.setTbSegment5("");
            }
            revenueConfig.setTbSegment6(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_6.getIndex())));
            if(StringUtils.isBlank(revenueConfig.getTbSegment6())){
                revenueConfig.setTbSegment6("");
            }
            String taxRat = POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_7.getIndex()));
            if(taxRat.indexOf("%")>0){
                taxRat = taxRat.replaceAll("%","");
                revenueConfig.setTaxRate(new BigDecimal(taxRat).divide(new BigDecimal(100)));
            }else{
                revenueConfig.setTaxRate(new BigDecimal(taxRat));
            }
            revenueConfig.setTaxBase(RevenueConfEnum.TaxBase.REVERSAL_MAPPING.get(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_8.getIndex()))));
            if(revenueConfig.getTaxBase().intValue() == 4){
                revenueConfig.setBaseDrCode(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_9.getIndex())));
                revenueConfig.setBaseCrCode("");
            }else if(revenueConfig.getTaxBase().intValue() == 5){
                revenueConfig.setBaseCrCode(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_9.getIndex())));
                revenueConfig.setBaseDrCode("");
            }else{
                revenueConfig.setBaseDrCode("");
                revenueConfig.setBaseCrCode("");
            }
            revenueConfig.setRevenueType(RevenueConfEnum.RevenueType.REVERSAL_MAPPING.get(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_10.getIndex()))));
            revenueConfig.setTaxType(RevenueConfEnum.TaxType.REVERSAL_MAPPING.get(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_11.getIndex()))));
            revenueConfig.setStatus(RevenueConfEnum.Status.REVERSAL_MAPPING.get(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_12.getIndex()))));
            revenueConfig.setStartDate(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_13.getIndex())));
            revenueConfig.setEndDate(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_14.getIndex())));
            revenueConfig.setCreateBy("Admin");
            revenueConfig.setUpdateBy("Admin");
            revenueConfig.setUpdateTime(new Date());
            revenueConfig.setCreateTime(new Date());
            dataList.add(revenueConfig);
        }
        if (1 == type) {
            if(CollectionUtils.isNotEmpty(orgIds)){
                RevenueConfigExample delExample = new RevenueConfigExample();
                delExample.createCriteria().andOrgIdIn(orgIds);
                revenueConfigMapper.deleteByExample(delExample);
            }
        }
        if (!CollectionUtils.isEmpty(dataList)) {
            List<List<RevenueConfig>> batchList = CommonUtils.subListWithLen(dataList, CommonUtils.BATCH_NUM_2000);
            batchList.forEach(l -> revenueConfigMapper.batchInsert(l));
        }
        return unSuccessList;
    }

    public String verifyColumn(Row row){
        String msg = "";
        if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_1.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_1.getName()+"值有错误|";
        }
        if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_7.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_7.getName()+"值有错误|";
        }
        if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_13.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_13.getName()+"值有错误|";
        }
        if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_14.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_14.getName()+"值有错误|";
        }
        if(!RevenueConfEnum.AccountType.REVERSAL_MAPPING.containsKey(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_3.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_3.getName()+"值有错误|";
        }else{
            if(RevenueConfEnum.AccountType.REVERSAL_MAPPING.get(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_3.getIndex()))).intValue()==1){
                if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_4.getIndex())))){
                    msg +="|"+RevenueConfEnum.EXECLColumn.Column_4.getName()+"值有错误|";
                }
                if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_5.getIndex())))){
                    msg +="|"+RevenueConfEnum.EXECLColumn.Column_5.getName()+"值有错误|";
                }
                if(StringUtils.isBlank(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_6.getIndex())))){
                    msg +="|"+RevenueConfEnum.EXECLColumn.Column_6.getName()+"值有错误|";
                }
            }
        }
        if(!RevenueConfEnum.TaxBase.REVERSAL_MAPPING.containsKey(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_8.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_8.getName()+"值有错误|";
        }
        if(!RevenueConfEnum.RevenueType.REVERSAL_MAPPING.containsKey(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_10.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_10.getName()+"值有错误|";
        }
        if(!RevenueConfEnum.TaxType.REVERSAL_MAPPING.containsKey(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_11.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_11.getName()+"值有错误|";
        }
        if(!RevenueConfEnum.Status.REVERSAL_MAPPING.containsKey(POIUtil.getCellFormulaString(row.getCell(RevenueConfEnum.EXECLColumn.Column_12.getIndex())))){
            msg +="|"+RevenueConfEnum.EXECLColumn.Column_12.getName()+"值有错误|";
        }
        return msg;
    }

    public Workbook export(InputStream is){
        Workbook tWorkbook = new XSSFWorkbook();
        try {
            tWorkbook = WorkbookFactory.create(is);
            Sheet sheet = tWorkbook.getSheetAt(0);
            Row sourceRow = sheet.getRow(1);
            //查询公司map
            List<OrgSelectDto> orgDtoList = organizationService.getMyOrgList();
            Map<String, String> orgMap = orgDtoList.stream().collect(Collectors.toMap(OrgSelectDto::getId, OrgSelectDto::getName, (value1, value2) -> {
                return value2;
            }));
            if(CollectionUtils.isNotEmpty(orgDtoList)){
                RevenueConfigExample example = new RevenueConfigExample();
                example.createCriteria().andOrgIdIn(orgDtoList.stream().map(OrgSelectDto::getId).collect(Collectors.toList()));
                example.setOrderByClause("org_id");
                List<RevenueConfig> dataList = revenueConfigMapper.selectByExample(example);
                if(CollectionUtils.isNotEmpty(dataList)){
                    for(int i = dataList.size()-1;i>=0;i--){
                        RevenueConfig revenueConfig = dataList.get(i);
                        Row row = POIUtil.createAndCloneRow(tWorkbook, sheet, 1, sourceRow);
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_1.getIndex()).setCellValue(revenueConfig.getName());
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_2.getIndex()).setCellValue(orgMap.get(revenueConfig.getOrgId()));
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_3.getIndex()).setCellValue(RevenueConfEnum.AccountType.MAPPING.get(revenueConfig.getAccountType()));
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_4.getIndex()).setCellValue(revenueConfig.getTbSegment3());
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_5.getIndex()).setCellValue(revenueConfig.getTbSegment5());
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_6.getIndex()).setCellValue(revenueConfig.getTbSegment6());
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_7.getIndex()).setCellValue(revenueConfig.getTaxRate().doubleValue());
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_8.getIndex()).setCellValue(RevenueConfEnum.TaxBase.MAPPING.get(revenueConfig.getTaxBase()));
                        if(revenueConfig.getTaxBase().intValue() == 4){
                            row.getCell(RevenueConfEnum.EXECLColumn.Column_9.getIndex()).setCellValue(revenueConfig.getBaseDrCode());
                        }else if(revenueConfig.getTaxBase().intValue() == 5){
                            row.getCell(RevenueConfEnum.EXECLColumn.Column_9.getIndex()).setCellValue(revenueConfig.getBaseCrCode());
                        }else{
                            row.getCell(RevenueConfEnum.EXECLColumn.Column_9.getIndex()).setCellValue("");
                        }
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_10.getIndex()).setCellValue(RevenueConfEnum.RevenueType.MAPPING.get(revenueConfig.getRevenueType()));
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_11.getIndex()).setCellValue(RevenueConfEnum.TaxType.MAPPING.get(revenueConfig.getTaxType()));
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_12.getIndex()).setCellValue(RevenueConfEnum.Status.MAPPING.get(revenueConfig.getStatus()));
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_13.getIndex()).setCellValue(revenueConfig.getStartDate());
                        row.getCell(RevenueConfEnum.EXECLColumn.Column_14.getIndex()).setCellValue(revenueConfig.getEndDate());
                    }
                }
            }
            sheet.removeRow(sourceRow);
            return tWorkbook;
        } catch (Exception e) {
            logger.error("export error.", e);
            throw Exceptions.SERVER_ERROR_EXCEPTION;
        }
    }
}
